extern "C" {
#include "robot.h"
}


// /* UTILITIES */


// // inline function to swap two numbers
void swap(char *x, char *y) {
	char t = *x; *x = *y; *y = t;
}

// function to reverse buffer[i..j]
char* reverse(char *buffer, int i, int j)
{
	while (i < j)
		swap(&buffer[i++], &buffer[j--]);
	return buffer;
}

// Iterative function to implement itoa() function in C
char* itoa(int value, char* buffer, int base)
{
	// invalid input
	if (base < 2 || base > 32)
		return buffer;
	// consider absolute value of number
	int n = value;
	int i = 0;
	while (n)
	{
		int r = n % base;
		if (r >= 10)
			buffer[i++] = 65 + (r - 10);
		else
			buffer[i++] = 48 + r;
		n = n / base;
	}
	// if number is 0
	if (i == 0)
		buffer[i++] = '0';
	// If base is 10 and value is negative, the resulting string
	// is preceded with a minus sign (-)
	// With any other base, value is always considered unsigned
	if (value < 0 && base == 10)
		buffer[i++] = '-';
	buffer[i] = '\0'; // null terminate string
	// reverse the string and return it
	return reverse(buffer, 0, i - 1);
}


int atoi(char *str)
{
    int res = 0; // Initialize result

    // Iterate through all characters of input string and
    // update result
    for (int i = 0; str[i] != '\0'; ++i)
        res = res*10 + str[i] - '0';

    // return result.
    return res;
}




/* ---- FONCTIONS D'ENVOI ---- */

// Stockage de la trame à envoyer
char packet[14];

// Calcul du checksum sur la trame à envoyer
int setChecksum(char * message) {
	int somme = 0;
	for (unsigned int i=0; i<strlen(message); i++) {
		somme += message[i];
	}
    //Caratere de fin
    somme += '-';
    somme += 'E';
    return somme;
}

// char * setupPacket(char * commande, char * params) {
  
//   char * packet = (char *)malloc(18 * sizeof(char));
//   strlcpy(packet, "S-", strlen("S-")+1);
//   strlcpy(packet + strlen(packet), commande, strlen(commande)+1);
//   strlcpy(packet + strlen(packet), "-", strlen("-")+1);
//   strlcpy(packet + strlen(packet), params, strlen(params)+1);
//   strlcpy(packet + strlen(packet), "-", strlen("-")+1);
//   int iChecksum = setChecksum(packet);
//   char cChecksum[5];
//   itoa(iChecksum, cChecksum, 10);
//   strlcpy(packet + strlen(packet), cChecksum, strlen(cChecksum)+1);
//   strlcpy(packet + strlen(packet), "-E", strlen("-E")+1);
//   return packet;
// }

// Formation de la trame à envoyer
void setupPacket(char * commande, char * params) {
  
  
  strlcpy(packet, "S-", strlen("S-")+1);
  strlcpy(packet + strlen(packet), commande, strlen(commande)+1);
  strlcpy(packet + strlen(packet), "-", strlen("-")+1);
  strlcpy(packet + strlen(packet), params, strlen(params)+1);
  strlcpy(packet + strlen(packet), "-", strlen("-")+1);
  int iChecksum = setChecksum(packet);
  char cChecksum[5];
  itoa(iChecksum, cChecksum, 10);
  strlcpy(packet + strlen(packet), cChecksum, strlen(cChecksum)+1);
  strlcpy(packet + strlen(packet), "-E", strlen("-E")+1);

}



/* ---- FONCTIONS DE RECEPTION V1 ---- */ 
/* Recevoir les valeurs des capteurs en une trame */


//Stockage de la trame reçue
const char * recu;
char copie[24];
char distance[4];
char lineSensorG[4];
char lineSensorC[4];
char lineSensorD[4];
char checksumRecu[5];


//Réception de trame
void lireTrame(void) {
  
  //while (strlen(recu) == 0) {
  	recu = mp_obj_str_get_str(readLine());
  //}
 }

//Découpage de la trame reçue
void parseTrame(void) {
	//Copie pour pouvoir vérifier le checksum après parsing
	strlcpy(copie, recu, 25);

	const char ch = '-';
	char * index;

	index = strchr(recu, ch);
	recu = index +1;
	strlcpy(distance, recu, 4);

	index = strchr(recu, ch);
	recu = index +1;
	strlcpy(lineSensorG, recu, 4); 

	index = strchr(recu, ch);
	recu = index +1;
	strlcpy(lineSensorC, recu, 4); 

	index = strchr(recu, ch);
	recu = index +1;
	strlcpy(lineSensorD, recu, 4); 

	index = strchr(recu, ch);
	recu = index +1;
	strlcpy(checksumRecu, recu, 5); 
}

//Vérification du checksum de la trame reçue
bool checksumOK(void) {
  int checksum = 0;
  for (int i=0; i<18;i++) {
    checksum += copie[i];
  }

  checksum += '-'; 
  checksum += 'E';
  

  if (checksum == atoi(checksumRecu)) {
    return true;
  }

  else {
    return false;
  }
}


//Attente de la réception d'un "OK" de l'Arduino
void acknowledgement(void) {
	char ack[] = "OK\n";
	const char * received;
	received = mp_obj_str_get_str(readLine());
	while(strcmp(received, ack) == 0) {
		received = mp_obj_str_get_str(readLine());
	}
}


/* ---- FONCTIONS DE RECEPTION V2 ---- */
/* Recevoir une trame par capteur */

// const char * recu;
// char copie[14];
// char commandeRecue[3];
// char paramsRecues[4];
// char checksumRecu[4];

// void lireTrame(void) {
  
//   //while (strlen(recu) == 0) {
//   	recu = mp_obj_str_get_str(readLine());
//   //}
//  }

// void parseTrame(void) {
// 	strlcpy(copie, recu, 15);
// 	const char ch = '-';
// 	char * index;
// 	index = strchr(recu, ch);
// 	recu = index +1;
// 	strlcpy(commandeRecue, recu, 3);

// 	index = strchr(recu, ch);
// 	recu = index +1;
// 	strlcpy(paramsRecues, recu, 4); 

// 	index = strchr(recu, ch);
// 	recu = index +1;
// 	strlcpy(checksumRecu, recu, 4); 
// }

// bool checksumOK(void) {
//   int checksum = 0;
//   for (int i=0; i<9;i++) {
//     checksum += copie[i];
//   }

//   checksum += '-'; 
//   checksum += 'E';
  

//   if (checksum == atoi(checksumRecu)) {
//     return true;
//   }

//   else {
//     return false;
//   }
// }

// void acknowledgement(void) {
// 	char ack[] = "OK\n";
// 	const char * received;
// 	received = mp_obj_str_get_str(readLine());
// 	while(strcmp(received, ack) == 0) {
// 		received = mp_obj_str_get_str(readLine());
// 	}
// }


/* ---- FONCTIONS DE COMMANDE V1---- */
/* Recevoir les valeurs des capteurs en une trame */

mp_obj_t ledVerte() {
	
	setupPacket((char*)LEDVERTE,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}

mp_obj_t ledRouge() {
	
	setupPacket((char*)LEDROUGE,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}

mp_obj_t moveForward() {

	setupPacket((char*)MOVE_FORWARD,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;

}


mp_obj_t moveBackward() {

	setupPacket((char*)MOVE_BACKWARD,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}

mp_obj_t moveLeft() {

	setupPacket((char*)MOVE_LEFT,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}

mp_obj_t moveRight() {

	setupPacket((char*)MOVE_RIGHT,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}

mp_obj_t stop() {

	setupPacket((char*)STOP,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	acknowledgement();
	return mp_const_none;
}


mp_obj_t measureSensors() {
	setupPacket((char*)MEASURE_SENSORS,(char*)"000");
	writeLine(mp_obj_new_str(packet,strlen(packet),false));
	lireTrame();
    parseTrame();
    if (checksumOK()) {
        return mp_const_none;
    }
    else {
    	ledRouge();
        measureSensors();
    } 
    return mp_const_none; 
}

mp_obj_t getDistanceSensorValue() {

    int value = atoi(distance);
    return mp_obj_new_int(value);
    
}


mp_obj_t getLineFollowerSensorGValue() {

    int value = atoi(lineSensorG);
    return mp_obj_new_int(value);
    
}

mp_obj_t getLineFollowerSensorCValue() {

    int value = atoi(lineSensorC);
    return mp_obj_new_int(value);
}

mp_obj_t getLineFollowerSensorDValue() {

    int value = atoi(lineSensorD);
    return mp_obj_new_int(value);

}


mp_obj_t getTemperatureSensorValue() {
	return mp_const_none;
}



/* ---- FONCTIONS DE COMMANDE V2---- */
/* Recevoir une trame par capteur */

// mp_obj_t ledVerte() {

// 	setupPacket((char*)LEDVERTE,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }

// mp_obj_t ledRouge() {

// 	setupPacket((char*)LEDROUGE,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }

// mp_obj_t moveForward() {

// 	setupPacket((char*)MOVE_FORWARD,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;

// }


// mp_obj_t moveBackward() {

// 	setupPacket((char*)MOVE_BACKWARD,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }

// mp_obj_t moveLeft() {

// 	setupPacket((char*)MOVE_LEFT,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }

// mp_obj_t moveRight() {

// 	setupPacket((char*)MOVE_RIGHT,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }

// mp_obj_t stop() {

// 	setupPacket((char*)STOP,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
// 	acknowledgement();
// 	return mp_const_none;
// }



// mp_obj_t getDistanceSensorValue() {

// 	setupPacket((char*)GET_DISTANCE,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
//     lireTrame();
//     parseTrame();
//     if (checksumOK()) {
//     	int value = atoi(paramsRecues);
//         return mp_obj_new_int(value);
//     }
//     else {
//     	ledRouge();
//     	getDistanceSensorValue();
//     }
//     return mp_const_none;    
// }


// mp_obj_t getLineFollowerSensorGValue() {

// 	setupPacket((char*)GET_LINEFOLLOWERG,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
//     lireTrame();
//     parseTrame();
//     if (checksumOK()) {
//     	int value = atoi(paramsRecues);
//         return mp_obj_new_int(value);
//     }
//     else {
//     	ledRouge();
//     	getLineFollowerSensorGValue();
//     }
//     return mp_const_none;
    
// }

// mp_obj_t getLineFollowerSensorCValue() {

// 	setupPacket((char*)GET_LINEFOLLOWERC,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
//     lireTrame();
//     parseTrame();

//     if (checksumOK()) {
//     	int value = atoi(paramsRecues);
//         return mp_obj_new_int(value);
//     }
//     else {
//     	ledRouge();
//     	getLineFollowerSensorCValue();
//     }
//     return mp_const_none;
// }

// mp_obj_t getLineFollowerSensorDValue() {

// 	setupPacket((char*)GET_LINEFOLLOWERD,(char*)"000");
// 	writeLine(mp_obj_new_str(packet,strlen(packet),false));
//  lireTrame();
//  parseTrame();
//  if (checksumOK()) {
//   int value = atoi(paramsRecues);
//      return mp_obj_new_int(value);
//  }
//  else {
//   ledRouge();
//   getLineFollowerSensorDValue();
//  }
//  return mp_const_none;
// }


// mp_obj_t getTemperatureSensorValue() {
// 	return mp_const_none;
// }
